#ifndef __Frisbee__
#define __Frisbee__

#include <cmath>
#include "Object3D.h"
#include "../Managers/MeshesManager.h"
#include "../Managers/SoundsManager.h"

#define FRISBEE_MODE_COUP_DROIT		0
#define FRISBEE_MODE_LOBE			1
#define FRISBEE_MODE_GHOST			2
#define FRISBEE_MODE_ENGAGEMENT		3
#define FRISBEE_MODE_LOST			4
#define FRISBEE_MODE_ON_THE_GROUND	5

#define FRISBEE_GRAVITY				7
#define FRISBEE_DEPTH_NORMAL		-4.9

class Frisbee : public Object3D
{
	protected:

		// SOUNDS
		Sound * bound;
		Sound * goal;
		Sound * lame;
		Sound * engage;

		// VALUES
		bool	visible;
		bool	trigo;
		int		mode;
		int		launcher;
		int		points;

	public:

		Frisbee( double _x, double _y, double _z, double _vx, double _vy, double _vz );

				void	init		();
		inline	int		getMode		() const;
		inline	int		getLauncher	() const;
		inline	void	setMode		( int _mode );
		inline	void	setLauncher	( int _mode );
		inline	int		getPoints	();
		inline	void	setVisible  ( bool );

		virtual void doEngagment(int _player);

		virtual void refresh( long temp );
		virtual void draw();

};


/**
 * Constructor
 */
Frisbee::Frisbee( double _x, double _y, double _z, double _vx, double _vy, double _vz ) : Object3D( _x,_y,_z,.2,.2,.2,90,0,90,_vx,_vy,_vz)
{
	mode	= FRISBEE_MODE_GHOST;
	visible = true;

	// MESHES
	MeshesManager   & m = MeshesManager	::getInstance();
	mesh				= m.addMesh("frisbee"	,"Meshes/Frisbee/frisbee.x");
	points				= 0;

	// SOUNDS
	SoundsManager   & snd = SoundsManager::getInstance();
	bound = snd.addSound("bound"	,"Sounds/bound.wav"	,3);
	goal  = snd.addSound("goal"		,"Sounds/goal.wav"	,1);
	lame  = snd.addSound("lame"		,"Sounds/lame.wav"	,1);
	engage= snd.addSound("engage"	,"Sounds/engage.wav",1);

	lame->setVolume(.3f);
	engage->setVolume(.5f);

	engage->play(-x/6.0,y,0);
}

/**
 * Initialise Frisbee values
 */
void Frisbee::init()
{
	x = y = z = vx = vy = vz = ry = launcher = 0;
}

/**
 * Get the frisbee mode
 */
int	Frisbee::getMode() const
{
	return mode;
}

/**
 * Set the frisbee mode
 */
void Frisbee::setMode( int _mode )
{
	mode = _mode;
}

/**
 * Get the identity of the player who throw the frisbee
 */
int	Frisbee::getLauncher() const
{
	return launcher;
}

/**
 * Modify the value of the player who throw the frisbee
 */
void Frisbee::setLauncher( int _player )
{
	launcher = _player;
}

/**
 * Return the points to the player
 */
int Frisbee::getPoints()
{
	int val = points;
	points  = 0;

	return val;
}

void Frisbee::doEngagment(int _player)
{
	int newx	= (_player*2-3) * 4;
	x			= y = z = vx = vy = vz = ry = 0;
	mode		= FRISBEE_MODE_ENGAGEMENT;
	x			= newx;
	points		= 0;
	launcher    = _player;

	engage->play(-x/6.0,y/4.0,0);
}

void Frisbee::setVisible  ( bool _visible )
{
	visible = _visible;
}

/**
 * Refresh the frisbee
 */
void Frisbee::refresh( long temp )
{
	KeyboardManager & key  = KeyboardManager::getInstance();

	// IF THE FRISBEE IS GIVEN BY AN ENGAGEMENT
	if( mode == FRISBEE_MODE_ENGAGEMENT )
	{
		vz -= FRISBEE_GRAVITY * temp / 500.0;

		// IF THE FRISBEE HAVE TOUCH THE FLOOR
		if( z < -5.3 )
		{
			z			= -5.3;
			vz			= 0;
			mode		= FRISBEE_MODE_ON_THE_GROUND;
			launcher	= 0;
		}
	}
	// IF MODE LOBE
	else if( mode == FRISBEE_MODE_LOBE )
	{
		vz -= FRISBEE_GRAVITY * temp / 500.0;

		// IF THE FRISBEE HAVE TOUCH THE FLOOR
		if( z < -5.3 )
		{
			z			= -5.3;
			vz			= 0;
			mode		= FRISBEE_MODE_LOST;
			points		= 2;

			lame->play();
		}
	}
	// IF THE FRISBEE IS LOST ( A PLAYER WON THE POINT )
	else if( mode == FRISBEE_MODE_LOST )
	{
		int newx	= (launcher*2-3) * 4;

		init();

		mode		= FRISBEE_MODE_ENGAGEMENT;
		x			= newx;

		engage->play(-x/6.0,y/4.0,0);
	}

	// IF THE FIRSBEE IN NOT INSIDE A ROBOT
	if( mode != FRISBEE_MODE_GHOST )
	{
		Object3D::refresh( temp );

		// COLLISIONS WITH THE ENVRONMENT
		if( y > 2.4 )
		{
			y = 2.4;
			vy = - vy;

			if( vx > 0 )
				trigo = true;
			else
				trigo = false;

			bound->play(-x/6.0,y/4.0,0);
		}
		else if( y <-2.4 )
		{
			y = -2.4;
			vy = - vy;

			if( vx > 0 )
				trigo = false;
			else
				trigo = true;	

			bound->play();
		}

		// A PLAYER WON
		if( x < -6.5 || x > 6.5 )
		{
			mode	= FRISBEE_MODE_LOST;

			if( y > -.6 && y < .6 )
				points	= 5;
			else
				points	= 3;

			goal->play();
		}

		// MAKE THE FRISBEE SPIN
		if( trigo  )
			ry += (sqrt(vx*vx)+sqrt(vy*vy))/5.0 * temp;
		else
			ry -= (sqrt(vx*vx)+sqrt(vy*vy))/5.0 * temp;
	}

}

/**
 * Draw the frisbee
 */
void Frisbee::draw()
{
	if( mode != FRISBEE_MODE_GHOST && visible )
		Object3D::draw();
}


#endif