#ifndef __Music__
#define __Music__

#include <al.h>
#include <alc.h>
#include <iostream>
#include <vector>
#include "sndfile.h"
#include "../Managers/OptionManager.h"

/*
 * BAUDOUX Jrme - TIXIER Benot
 */
class Music
{
	public:
		ALuint buffer;
		ALuint source;
		int nbSources;

		Music( const std::string & file );
		~Music();

		void play();
		void stop();
		void pause();
		void resume();
		bool isPlayed();
		void setVolume( float );

	private:
		ALuint LoadSound(const std::string& Filename);
};

/**
 * Constructor
 * @param file file
 */
Music::Music(  const std::string & file )
{
	// EN DUR POUR L'INSTANT
	buffer	= LoadSound(file);
	alGenSources(1, &source);
	alSourcei(source, AL_LOOPING,  AL_TRUE  );
	alSourcei(source, AL_BUFFER, buffer);
}

/**
 * Destructor
 */
Music::~Music()
{

}

/**
 * Play the sound
 */
void Music::play()
{
	ALint Status;

	alGetSourcei(source, AL_SOURCE_STATE, &Status);
	if( Status != AL_PLAYING )
	{
		alSourcePlay(source);
	}

}

/**
 * Stop the sound
 */
void Music::stop()
{
	alSourceStop(source);
}

/**
 * Pause the sound
 */
void Music::pause()
{
	alSourcePause(source);
}

/**
 * Resume the sound
 */
void Music::resume()
{
	alSourcePlay(source);
}

/**
 * Return true if the source is currently playing
 * @return true if the source is currently playing
 */
bool Music::isPlayed()
{
	ALint Status;
	alGetSourcei(source, AL_SOURCE_STATE, &Status);
	if( Status == AL_PLAYING )
		return true;
	return false;
}

/**
 * Change the gain
 * @param vol gain
 */
void Music::setVolume( float vol )
{
	alSourcef(source, AL_GAIN, vol);
}

/**
 * Private function to load a wav file
 */
ALuint Music::LoadSound(const std::string& Filename)
{
    // Ouverture du fichier audio avec libsndfile
    SF_INFO FileInfos;
    SNDFILE* File = sf_open(Filename.c_str(), SFM_READ, &FileInfos);
    if (!File)
    {
        std::cerr << "Impossible d'ouvrir le fichier audio" << std::endl;
        return 0;
    }

    // Lecture du nombre d'chantillons et du taux d'chantillonnage (nombre d'chantillons  lire par seconde)
    ALsizei NbSamples  = static_cast<ALsizei>(FileInfos.channels * FileInfos.frames);
    ALsizei SampleRate = static_cast<ALsizei>(FileInfos.samplerate);

    // Lecture des chantillons audio au format entier 16 bits sign (le plus commun)
    std::vector<ALshort> Samples(NbSamples);
    if (sf_read_short(File, &Samples[0], NbSamples) < NbSamples)
    {
        std::cerr << "Impossible de lire les chantillons stocks dans le fichier audio" << std::endl;
        return 0;
    }

    // Fermeture du fichier
    sf_close(File);

    // Dtermination du format en fonction du nombre de canaux
    ALenum Format;
    switch (FileInfos.channels)
    {
        case 1 : Format = AL_FORMAT_MONO16;   break;
        case 2 : Format = AL_FORMAT_STEREO16; break;
        default :
            std::cerr << "Format audio non support (plus de 2 canaux)" << std::endl;
            return 0;
    }

    // Cration du tampon OpenAL
    ALuint Buffer;
    alGenBuffers(1, &Buffer);

    // Remplissage avec les chantillons lus
    alBufferData(Buffer, Format, &Samples[0], NbSamples * sizeof(ALushort), SampleRate);

    // Vrification des erreurs
    if (alGetError() != AL_NO_ERROR)
    {
        std::cerr << "Impossible de remplir le tampon OpenAL avec les chantillons du fichier audio" << std::endl;
        return 0;
    }

    return Buffer;
}

#endif