#ifndef __Sound__
#define __Sound__

#include <al.h>
#include <alc.h>
#include <iostream>
#include <vector>
#include "sndfile.h"
#include "../Managers/OptionManager.h"

/**
 * a Mesh
 */
class Sound
{
	public:
		ALuint buffer;
		ALuint * source;
		int nbSources;

		Sound( const std::string & file, int nbEch, bool );
		~Sound();

		inline void play(double,double,double);
		inline void playSource( int, double, double, double);

		inline void stop();
		inline void stopSource(int);

		inline void pause();
		inline void pauseSource(int);

		inline void resume();
		inline void resumeSource(int);

		inline bool isPlayed( int );

		inline void setVolume( float );
		inline void setVolumeSource( int,float );

		inline void setSpeed( double );
		inline void setSpeedSource( int, double );

		inline void setPositionSource( int, double, double, double );


	private:
		ALuint LoadSound(const std::string& Filename);
};

/**
 * Constructor
 * @param file file name
 * @param nbEch number of buffers
 * @param loop is it looping
 */
Sound::Sound(  const std::string & file, int nbEch=1, bool loop=false )
{
	source		= new ALuint[nbEch];
	nbSources	= nbEch;

	buffer	= LoadSound(file);

	// LOAD THE SOUNDS
	for( int i=0; i<nbEch; i++ )
	{
		// EN DUR POUR L'INSTANT
		alGenSources(1, &source[i]);

		if( loop )
			alSourcei(source[i], AL_LOOPING,  AL_TRUE  );

		alSourcei(source[i], AL_BUFFER, buffer);
	}


}

/**
 * Destructor
 */
Sound::~Sound()
{
	if( source != NULL )
	{
		delete[] source;
		source = NULL;
	}
}

/**
 * Play the sound
 * @param x position x of the sound
 * @param y position y of the sound
 * @param z position z of the sound
 */
void Sound::play( double x=0, double y=0, double z=0)
{
	ALint Status;
	bool lect = false;

	for( int i=0; i<nbSources && !lect; i++)
	{
		alGetSourcei(source[i], AL_SOURCE_STATE, &Status);
		if( Status != AL_PLAYING )
		{
			lect = true;
			alSource3f(source[i],AL_POSITION, (float)x, (float)y, (float)z);
			alSourcePlay(source[i]);
		}
	}

	// IF NO TRACK AVAILABLE ... READ THE 1ST TRACK
	if( !lect )
		alSourcePlay(source[0]);
}

/**
 * Play the sound
 * @param src source to play
 * @param x position x of the sound
 * @param y position y of the sound
 * @param z position z of the sound
 */
void Sound::playSource( int src, double x=0, double y=0, double z=0)
{
	alSource3f(source[src],AL_POSITION, (float)x, (float)y, (float)z);
	alSourcePlay(source[src]);
}

/**
 * Stop the sound
 */
void Sound::stop()
{
	for( int i=0; i<nbSources; i++)
		alSourceStop(source[i]);
}

/**
 * Stop the sound
 * @param src source to stop
 */
void Sound::stopSource(int src)
{
	alSourceStop(source[src]);
}

/**
 * Pause the sound
 */
void Sound::pause()
{
	for( int i=0; i<nbSources; i++)
		alSourcePause(source[i]);
}

/**
 * Pause the sound
 * @param src source to stop
 */
void Sound::pauseSource(int src)
{
	alSourcePause(source[src]);
}

/**
 * Resume the sound
 */
void Sound::resume()
{
	ALint Status;
	for( int i=0; i<nbSources; i++)
	{
		alGetSourcei(source[i], AL_SOURCE_STATE, &Status);
		if( Status != AL_PAUSED )
			alSourcePlay(source[i]);
	}
}

/**
 * Resume the sound
 * @param src source to resume
 */
void Sound::resumeSource(int src)
{
	ALint Status;

	alGetSourcei(source[src], AL_SOURCE_STATE, &Status);
	if( Status != AL_PAUSED )
		alSourcePlay(source[src]);
}

/**
 * Return true if the source is currently playing
 * @param i test the source i
 * @return true if the source is currently playing
 */
bool Sound::isPlayed( int i )
{
	ALint Status;
	alGetSourcei(source[i], AL_SOURCE_STATE, &Status);
	if( Status == AL_PLAYING )
		return true;
	return false;
}

/**
 * Modify the gain of all sources of that sound
 * @param vol gain
 */
void Sound::setVolume( float vol )
{
	for( int i=0; i<nbSources; i++)
		alSourcef(source[i], AL_GAIN, vol);

}

/**
 * Modify the gain of all sources of that sound
 * @param src modify the volume of that source
 * @param vol gain
 */
void Sound::setVolumeSource( int src, float vol )
{
	alSourcef(source[src], AL_GAIN, vol);
}

/**
 * Modify the speed of the sound
 * @param speed speed of the sound
 */
void Sound::setSpeed( double speed = 0)
{
	for( int i=0; i<nbSources; i++)
		alSourcef(source[i],AL_PITCH, (float)speed);
}

/**
 * Modify the speed of the source
 * @src number of the source
 * @param speed speed of the sound
 */
void Sound::setSpeedSource( int src, double speed = 0)
{
	alSourcef(source[src],AL_PITCH, (float)speed);
}

/**
 * Modify the position of the sound
 * @src number of the source
 * @param x position x of the sound
 * @param y position y of the sound
 * @param z position z of the sound
 */
void Sound::setPositionSource( int src, double x=0, double y=0, double z=0)
{
	alSource3f(source[src],AL_POSITION, (float)x, (float)y, (float)z);
}

/**
 * Private function to load a wav file
 * @param Filename file name
 * @return id of the source
 */
ALuint Sound::LoadSound(const std::string& Filename)
{
    // Ouverture du fichier audio avec libsndfile
    SF_INFO FileInfos;
    SNDFILE* File = sf_open(Filename.c_str(), SFM_READ, &FileInfos);
    if (!File)
    {
        std::cerr << "Impossible d'ouvrir le fichier audio" << std::endl;
        return 0;
    }

    // Lecture du nombre d'chantillons et du taux d'chantillonnage (nombre d'chantillons  lire par seconde)
    ALsizei NbSamples  = static_cast<ALsizei>(FileInfos.channels * FileInfos.frames);
    ALsizei SampleRate = static_cast<ALsizei>(FileInfos.samplerate);

    // Lecture des chantillons audio au format entier 16 bits sign (le plus commun)
    std::vector<ALshort> Samples(NbSamples);
    if (sf_read_short(File, &Samples[0], NbSamples) < NbSamples)
    {
        std::cerr << "Impossible de lire les chantillons stocks dans le fichier audio" << std::endl;
        return 0;
    }

    // Fermeture du fichier
    sf_close(File);

    // Dtermination du format en fonction du nombre de canaux
    ALenum Format;
    switch (FileInfos.channels)
    {
        case 1 : Format = AL_FORMAT_MONO16;   break;
        case 2 : Format = AL_FORMAT_STEREO16; break;
        default :
            std::cerr << "Format audio non support (plus de 2 canaux)" << std::endl;
            return 0;
    }

    // Cration du tampon OpenAL
    ALuint Buffer;
    alGenBuffers(1, &Buffer);

    // Remplissage avec les chantillons lus
    alBufferData(Buffer, Format, &Samples[0], NbSamples * sizeof(ALushort), SampleRate);

    // Vrification des erreurs
    if (alGetError() != AL_NO_ERROR)
    {
        std::cerr << "Impossible de remplir le tampon OpenAL avec les chantillons du fichier audio" << std::endl;
        return 0;
    }

    return Buffer;
}

#endif