#ifndef __GameManager__
#define __GameManager__

// C++
#include <iostream>
#include <fstream>
#include <sstream>
// MANAGERS
#include "Singleton.h"
#include "OptionManager.h"
#include "SoundsManager.h"
#include "ControlsManager.h"
// ELEMENTS
#include "../Elements/Object2D.h"
#include "../Elements/Object3D.h"
#include "../Elements/Robot.h"
#include "../Elements/Frisbee.h"
#include "../Elements/Notification.h"
// MENUS
#include "../Menu/MainMenu.h"
#include "../Menu/MenuAbout.h"
#include "../Menu/MenuIntro.h"
#include "../Menu/MenuPause.h"
#include "../Menu/MenuOptions.h"
#include "../Menu/MenuCharacters.h"

#define GAMEMANAGER_POINTS_PER_SETS	20
#define GAMEMANAGER_SETS_BY_GAME	2
#define GAMEMANAGER_TIME_GAME		99000

enum
{
	GAMEMANAGER_GAME_OK,
	GAMEMANAGER_SET_WON,
	GAMEMANAGER_GAME_WON,
	GAMEMANAGER_GAME_OVER,
	GAMEMANAGER_TIMES_UP,
	GAMEMANAGER_TIMES_UP_OK
};

/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class GameManager : public Singleton<GameManager>
{
	private:
		friend class Singleton<GameManager>; 

		// MENUS
		Menu		 * menu;
		Notification * notification;

		// VALUES
		int  points[2];
		int  victories[2];
		int  status;
		long time;

		// OBJETS
		Object3D	 * arena;
		Robot		 * robot1;
		Robot		 * robot2;
		Frisbee		 * frisbee;

		// SOUNDS
		Sound* beep2;
		Sound* musicGame;
		Sound* musicMenu;
		Sound* musicIntro;

		// MANAGERS
		SpritesManager  * sp;
		MeshesManager   * mesh;
		TextManager     * text;
		OptionManager   * op;
		ControlsManager * ctrl;
		SoundsManager   * snd;

	public :
		~GameManager();
		void init();
		void refresh			( long );
		void draw();

	private:
		GameManager();
		void reset( int );
		void destroy();

		void refreshMenu		( long );
		void refreshGame		( long );
		void refreshNotification( long );

};

/**
 * Private constructor
 */
GameManager::GameManager()
{
	sp   = &SpritesManager	::getInstance();
	mesh = &MeshesManager	::getInstance();
	text = &TextManager		::getInstance();
	op   = &OptionManager	::getInstance();
	ctrl = &ControlsManager	::getInstance();
	snd  = &SoundsManager	::getInstance();

	arena			= NULL;
	robot1			= NULL;
	robot2			= NULL;
	frisbee			= NULL;
	notification	= NULL;
}

/**
 * Initiate the game
 */
void GameManager::init()
{
	// ADD THE RESSOURCES
	menu		= new MenuIntro();

	// FONTS
	text->addFont("arial"	,"Arial",25,0,false,FW_NORMAL);
	text->addFont("arialBig","Arial",35,0,false,FW_NORMAL);

	// SOUNDS
	beep2		= snd->addSound("beep2"		 ,"Sounds/beep2.wav",5);
	musicGame	= snd->addSound("musicGame"	 ,"Musics/musicGame.wav" ,1,true);
	musicMenu	= snd->addSound("musicMenu"	 ,"Musics/musicMenu.wav" ,1,true);
	musicIntro	= snd->addSound("musicIntro" ,"Musics/musicIntro.wav",1,false);

	musicGame->setVolume(.8f);

	// MESHES
	mesh->addMesh("arena","Meshes/Arena/arena.x");

	// TEXTURES
	sp->addSprite("5points"		,"Images/Interface/5points.png");
	sp->addSprite("3points"		,"Images/Interface/3points.png");
	sp->addSprite("2points"		,"Images/Interface/2points.png");
	sp->addSprite("p1WonSet"	,"Images/Interface/p1WonSet.png");
	sp->addSprite("p2WonSet"	,"Images/Interface/p2WonSet.png");
	sp->addSprite("p1WonGame"	,"Images/Interface/p1WonGame.png");
	sp->addSprite("p2WonGame"	,"Images/Interface/p2WonGame.png");
	sp->addSprite("draw"		,"Images/Interface/draw.png");
	sp->addSprite("timesup"		,"Images/Interface/timesup.png");


	// START MUSIC
	musicIntro ->play();
}


/**
 * Initiate the game
 */
void GameManager::reset( int state )
{
	destroy();

	// RESET THE POINTS
	points[0]	= 0;
	points[1]	= 0;

	// RESET THE VICTORIES
	victories[0]= 0;
	victories[1]= 0;

	time = GAMEMANAGER_TIME_GAME;

	// STATUS
	status = GAMEMANAGER_GAME_OK;

	// OBJECTS
	arena	= new Object3D(0,.03,-5.5,2.2,1.5,1.5,90,90,0,0,0,0);
	arena  ->setMesh( mesh->getMesh("arena") );
	frisbee = new Frisbee(4,0,0,0,0,0);
	frisbee->setMode( FRISBEE_MODE_ENGAGEMENT );

	// Extract the model that wants the player from the state
	int player1 = (state - ((int)(state/10))*10) - 1;
	int player2 = (int)(state/10) - 1;

	robot1	= new Robot(6 ,0,1,player1);
	robot2	= new Robot(-6,0,2,player2);
}

/**
 * public Destructor
 */
GameManager::~GameManager()
{
	destroy();
}

/**
 * Desttroy all the dynamics objects
 */
void GameManager::destroy()
{
	if( menu	!= NULL )
	{
		delete menu;
		menu = NULL;
	}

	if( arena	!= NULL )
	{
		delete arena;
		arena = NULL;
	}

	if( robot1	!= NULL )
	{
		delete robot1;
		robot1 = NULL;
	}

	if( robot2	!= NULL )
	{
		delete robot2;
		robot2 = NULL;
	}

	if( frisbee	!= NULL )
	{
		delete frisbee;
		frisbee = NULL;
	}
}

/**
 * Refresh the menu
 * @param temps time bewteen two frames
 */
void GameManager::refreshMenu( long temps )
{
	menu->refresh(temps);

	// IF WE ARE IN THE INTRO MENU
	if( menu->getName() == std::string("MenuIntro") )
	{
		if( menu->getCode() == MENUINTRO_GO_PRINCIPAL )
		{
			delete menu;
			menu = new MainMenu();

			musicIntro ->stop();
			musicMenu  ->play();

			// KILL THE SOUNDS
			snd->removeSound("musicIntro");
		}
	}
	// IF WE ARE IN THE MAIN MENU
	else if( menu->getName() == std::string("MainMenu") )
	{
		// RUN THE GAME
		if( menu->getCode() == MAINMENU_PLAY )
		{
			delete menu;
			menu = new MenuCharacters();
		}
		// GO TO OPTIONS
		else if( menu->getCode() == MAINMENU_OPTIONS )
		{
			delete menu;
			menu = new MenuOptions();
		}
		// GO TO ABOUT
		else if( menu->getCode() == MAINMENU_ABOUT )
		{
			delete menu;
			menu = new MenuAbout();
		}
		// EXIT THE GAME
		else if( menu->getCode() == MAINMENU_EXIT )
		{
			exit(0);
		}
	}
	// IF WE ARE IN THE MAIN MENU
	else if( menu->getName() == std::string("Character") )
	{
		if( menu->getCode() == MENUCHARACTERS_BACK )
		{
			delete menu;
			menu = new MainMenu();
		}
		else if( ! menu->getCode() == MENUCHARACTERS_CONTINUE )
		{
			reset(menu->getCode());
			musicMenu->stop();
			musicGame->play();
		}
	}
	// IF WE ARE IN THE OPTIONS MENU
	else if( menu->getName() == std::string("Options") )
	{
		// GO BACK TO MAIN MENU
		if( menu->getCode() == MENUOPTIONS_BACK )
		{
			delete menu;
			menu = new MainMenu();
		}
	}
	// IF WE ARE IN THE ABOUT MENU
	else if( menu->getName() == std::string("About") )
	{
		// GO BACK TO MAIN MENU
		if( menu->getCode() == MENUABOUT_BACK )
		{
			delete menu;
			menu = new MainMenu();
		}
	}
	// IF WE ARE IN THE PAUSE MENU
	else if( menu->getName() == std::string("Pause") )
	{
		// CONTINUE THE GAME
		if( menu->getCode() == MENUPAUSE_RETURN )
		{
			delete menu;
			menu = NULL;

			musicMenu->stop();
			musicGame->play();
		}
		// EXIT TO MAIN MENU
		else if( menu->getCode() == MENUPAUSE_EXIT )
		{
			delete menu;
			menu = new MainMenu();
		}
	}
}

/**
 * Refresh The game
 * @param temps time bewteen two frames
 */
void GameManager::refreshGame( long temps )
{
	// REFRESH ALL THE ELEMENTS
	robot1	->refresh(temps);
	robot2	->refresh(temps);
	frisbee	->refresh(temps);

	// DETECTS AND PERFORM THE COLLISIONS
	robot1 ->collision( frisbee );
	robot2 ->collision( frisbee );

	// THE TIME PASSES IS THE FRISBEE IS NOT ON THE GROUND
	if( frisbee->getMode() != FRISBEE_MODE_ENGAGEMENT && frisbee->getMode() != FRISBEE_MODE_ON_THE_GROUND )
		time -= temps;

	// IF A PLAYER GOALS ... MODIFY THE POINTS
	int pointsWin = frisbee->getPoints();
	if( pointsWin > 0 )
	{
		// MODIFY THE POINTS
		points[ (int) frisbee->getLauncher() - 1 ] += pointsWin;

		// CREATE THE POINT NOTIFICATION
		notification = new Notification( op->getScreenWidth()/4, op->getScreenHeight()/4, op->getScreenWidth()/2, op->getScreenHeight()/2 );
		if( pointsWin == 5 )
			notification->setSprite( sp->getSprite("5points") );
		else if( pointsWin == 3 )
			notification->setSprite( sp->getSprite("3points") );
		else if( pointsWin == 2 )
			notification->setSprite( sp->getSprite("2points") );

		// ... GO TO THE "// IF THE NOTIFICATION IS OVER" CAUSE NOTHING WILL APPEND TILL THEN
	}

	// IF THE TIME IS OVER
	if( time <= 0 )
	{
		time = 0;
		notification = new Notification( op->getScreenWidth()/4, op->getScreenHeight()/4, op->getScreenWidth()/2, op->getScreenHeight()/2 );
		notification->setSprite( sp->getSprite("timesup") );
		status = GAMEMANAGER_TIMES_UP;
		frisbee->setVisible(false);
	}

	// MENU PAUSE
	if( ctrl->isEscape() )
	{
		ctrl->initEscape();
		menu = new MenuPause();

		beep2->play();

		musicGame->stop();
		musicMenu->play();
	}
}

/**
 * Refresh The notification
 * @param temps time bewteen two frames
 */
void GameManager::refreshNotification( long temps )
{
	// REFRESH THE NOTIFICATION
	notification->refresh(temps);

	// IF THE NOTIFICATION IS OVER
	if( notification->isFinished() )
	{
		// DELETE THE NOTIFICATION
		delete notification;
		notification = NULL;

		// REPLACE THE ROBOTS
		robot1->replace( 6,0);
		robot2->replace(-6,0);

		// VERIFY IF THE PLAYER WON THE SET
		if( points[ (int) frisbee->getLauncher() - 1 ] >= GAMEMANAGER_POINTS_PER_SETS )
		{
			victories[ (int) frisbee->getLauncher() - 1 ] ++;
			points[0] = 0;
			points[1] = 0;
			status = GAMEMANAGER_SET_WON;
		}	

		// LOOKS THE STATUS
		switch( status )
		{
			// IF THE LAST SET IS TIME'S UP
			case GAMEMANAGER_TIMES_UP:
			{
				notification = new Notification( op->getScreenWidth()/4, op->getScreenHeight()/4, op->getScreenWidth()/2, op->getScreenHeight()/2 );
				notification->setTime(2000);

				// IF THE PLAYER ONE HAS MORE POINTS THAN THE PLAYER TWO
				if( points[0] > points[1] )
				{
					notification->setSprite( sp->getSprite("p1WonSet") );
					victories[0]++;

					frisbee->doEngagment(1);
				}
				// IF THE PLAYER TWO HAS MORE POINTS THAN THE PLAYER ONE
				else if( points[0] < points[1] )
				{
					notification->setSprite( sp->getSprite("p2WonSet") );
					victories[1]++;

					frisbee->doEngagment(2);
				}
				// IF DRAW
				else
				{
					notification->setSprite( sp->getSprite("draw") );

					// IF THE FIRST PLAYER HAS MORE SETS ... LET THE OTHER BEGIN
					if( victories[0] > victories[1] )
						frisbee->doEngagment(2);
					else
						frisbee->doEngagment(1);
				}

				// VERIFY IF THE PLAYER WON THE GAME
				if( victories[0] >= GAMEMANAGER_SETS_BY_GAME )
				{
					status = GAMEMANAGER_GAME_WON;
					frisbee->setLauncher(1);
				}
				else if( victories[1] >= GAMEMANAGER_SETS_BY_GAME )
				{
					status = GAMEMANAGER_GAME_WON;
					frisbee->setLauncher(2);
				}
				else
				{
					status	= GAMEMANAGER_TIMES_UP_OK;
				}

				// RESETS VALUES
				points[0]	= 0;
				points[1]	= 0;
				time		= GAMEMANAGER_TIME_GAME;

				break;
			}

			// IF THE FRISBEE IS ENGAGED AFTER A TIME'S UP .. SET THE FRISBEE VISIBLE
			case GAMEMANAGER_TIMES_UP_OK:
			{
				frisbee->setVisible(true);
				break;
			}

			// IF THE PLAYER WON THE SET
			case GAMEMANAGER_SET_WON:
			{
				notification = new Notification( op->getScreenWidth()/4, op->getScreenHeight()/4, op->getScreenWidth()/2, op->getScreenHeight()/2 );
				
				// INDICATE WHO WON THE SET
				if( frisbee->getLauncher() == 1 )
					notification->setSprite( sp->getSprite("p1WonSet") );
				else
					notification->setSprite( sp->getSprite("p2WonSet") );

				// SET THE TIME
				notification->setTime(2000);

				// VERIFY IF THE PLAYER WON THE GAME
				if( victories[ (int) frisbee->getLauncher() - 1 ] >= GAMEMANAGER_SETS_BY_GAME )
				{
					points[0] = 0;
					points[1] = 0;

					status = GAMEMANAGER_GAME_WON;
				}	
				else
					status	= GAMEMANAGER_GAME_OK;

				break;
			}

			// IF THE PLAYER WON THE GAME
			case GAMEMANAGER_GAME_WON:
			{
				notification = new Notification( op->getScreenWidth()/4, op->getScreenHeight()/4, op->getScreenWidth()/2, op->getScreenHeight()/2 );
				
				// INDICATE WHO WON THE GAME
				if( frisbee->getLauncher() == 1 )
					notification->setSprite( sp->getSprite("p1WonGame") );
				else
					notification->setSprite( sp->getSprite("p2WonGame") );

				// SET THE TIME
				notification->setTime(3000);

				status	= GAMEMANAGER_GAME_OVER;
				break;
			}

			// GAME OVER WE CLOSE THE GAME
			case GAMEMANAGER_GAME_OVER:
			{
				// GO TO MAIN MENU
				menu	= new MainMenu();
				musicGame->stop();
				musicMenu->play();
				break;
			}
		}
	}
}



/**
 * Refresh the game elements
 @param temps time weteween two frames
 */
void GameManager::refresh( long temps )
{
	if( menu == NULL )
	{
		if( notification == NULL )
		{
			refreshGame(temps);
		}
		else
		{
			refreshNotification(temps);
		}
	}
	else
	{
		refreshMenu(temps);
	}
}


/**
 * Draw the game elements
 */
void GameManager::draw()
{
	// IF THE GAME IS ON
	if( menu == NULL )
	{
		// MESHES
		arena  ->draw();
		robot1 ->draw();
		robot2 ->draw();
		frisbee->draw();

		// SPRITES
		sp	->getSpriteDevice()->Begin(NULL);

			if( notification != NULL )
				notification->draw();

		sp	->getSpriteDevice()->End();

		// TEXTES
		std::stringstream score1;
		std::stringstream score2;
		std::stringstream timeStr;

		score1 << "Player One : " << points[0] << " Pts - " << victories[0] << " sets";
		score2 << "Player Two : " << points[1] << " Pts - " << victories[1] << " sets";
		timeStr<< "Time\n" << (int)time/1000;

		text->drawText("arial"		,score1.str().c_str()	, 30,10,330,200,255,255,255,255);
		text->drawText("arial"		,score2.str().c_str()	,(int)(op->getScreenWidth() - 330)		, 10, (int)(op->getScreenWidth() - 30)		,200,255,255,255,255, DT_RIGHT | DT_TOP);
		text->drawText("arialBig"	,timeStr.str().c_str()	,(int)(op->getScreenWidth()/2.0 - 100)	, 10, (int)(op->getScreenWidth()/2.0 +100)	,200,255,255,255,255, DT_CENTER | DT_TOP);
	}
	// IF THE MENU IS ON
	else
	{
		// SPRITES
		sp	->getSpriteDevice()->Begin(NULL);
		menu->draw2D();
		sp	->getSpriteDevice()->End();
		menu->draw();
	}

}


#endif