#ifndef __JOYPADMANAGER__
#define __JOYPADMANAGER__

#define JOYPADMANAGER_NB_JOYPADS	2

#include <dinput.h>
#include "Singleton.h"

class JoypadManager : public Singleton<JoypadManager>
{
	private:
		friend class Singleton<JoypadManager>; 

	public:
		HWND					hwnd;
		LPDIRECTINPUT8			din;
		LPDIRECTINPUTDEVICE8	dinDevice;
		LPDIRECTINPUTDEVICE8	dinPaddle[JOYPADMANAGER_NB_JOYPADS];
		DIJOYSTATE				paddle[JOYPADMANAGER_NB_JOYPADS];
		int						nbPaddles;

		JoypadManager();
		~JoypadManager();

		void refresh();

		inline bool isUp    ( int player );
		inline bool isDown  ( int player );
		inline bool isLeft  ( int player );
		inline bool isRight ( int player );
		inline bool isButton( int button, int player );

		bool WINAPI init();
		void clean();
};


BOOL CALLBACK EnumJoysticksCallback (const DIDEVICEINSTANCE* pdidInstance, VOID* pContext);
BOOL CALLBACK EnumAxesCallback      (const DIDEVICEOBJECTINSTANCE* pdidoi, VOID* pContext);

/**
 * Private constructor
 */
JoypadManager::JoypadManager()
{
	nbPaddles = 0;

	dinPaddle[0] = NULL;
	dinPaddle[1] = NULL;
}

/**
 * Public Destructor
 */
JoypadManager::~JoypadManager()
{
	clean();
}

/**
 * True if the pov is up
 */
bool JoypadManager::isUp( int player )
{
	if( nbPaddles >= player )
		return paddle[player-1].rgdwPOV[0] == 0 || paddle[player-1].rgdwPOV[0] == 4500 || paddle[player-1].rgdwPOV[0] == 31500;
	else
		return false;
}

/**
 * True if the pov is down
 */
bool JoypadManager::isDown( int player )
{
	if( nbPaddles >= player )
		return paddle[player-1].rgdwPOV[0] == 18000 || paddle[player-1].rgdwPOV[0] == 22500 || paddle[player-1].rgdwPOV[0] == 13500;
	else
		return false;
}

/**
 * True if the pov is left
 */
bool JoypadManager::isLeft( int player )
{
	if( nbPaddles >= player )
		return paddle[player-1].rgdwPOV[0] == 27000 || paddle[player-1].rgdwPOV[0] == 22500 || paddle[player-1].rgdwPOV[0] == 31500;
	else
		return false;
}

/**
 * True if the pov is right
 */
bool JoypadManager::isRight( int player )
{
	if( nbPaddles >= player )
		return paddle[player-1].rgdwPOV[0] == 9000 || paddle[player-1].rgdwPOV[0] == 4500 || paddle[player-1].rgdwPOV[0] == 13500;
	else
		return false;
}

/**
 * True if the B1 is pressed
 */
bool JoypadManager::isButton( int button, int player )
{
	if( nbPaddles >= player )
		return paddle[player-1].rgbButtons[button-1] != 0;
	else
		return false;
}



/**
 * Refresh the joystick(s)
 */
void JoypadManager::refresh()
{
	for( int i=0; i<nbPaddles; i++)
	{
		ZeroMemory(&paddle[i], sizeof(DIJOYSTATE));
		dinPaddle[i]->Poll();
		dinPaddle[i]->GetDeviceState(sizeof(DIJOYSTATE),&paddle[i]);
	}

}

/**
 * Init the joystick(s)
 */ 
bool WINAPI JoypadManager::init ()
{
	// INIT
	DirectXManager & dx = DirectXManager::getInstance();
	din					= dx.din;
	dinDevice			= dx.dinDevice;
	hwnd				= dx.hWnd;
	HRESULT	hr;

	// PADDLES
	hr=din->EnumDevices(DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback, NULL, DIEDFL_ATTACHEDONLY);

	return true;
}


/**
 * Enum Joysticks Callback
 */
BOOL CALLBACK EnumJoysticksCallback (const DIDEVICEINSTANCE* pdidInstance, VOID* pContext)
{
	JoypadManager &joy = JoypadManager::getInstance();

	HRESULT hr;

	// STOP THE READING IF ENOUGTH JOYPAD ARE ALREADY ADDED
	if( joy.nbPaddles < JOYPADMANAGER_NB_JOYPADS )
	{
		hr=joy.din->CreateDevice(pdidInstance->guidInstance, &joy.dinPaddle[joy.nbPaddles], NULL);

		if FAILED (hr)	
			return DIENUM_STOP;

		hr=joy.dinPaddle[joy.nbPaddles]->SetDataFormat(&c_dfDIJoystick);
		if (FAILED (hr))	
			return DIENUM_STOP;

		hr=joy.dinPaddle[joy.nbPaddles]->SetCooperativeLevel(joy.hwnd, DISCL_FOREGROUND|DISCL_EXCLUSIVE);
		if (FAILED (hr))	
			return DIENUM_STOP;

		hr=joy.dinPaddle[joy.nbPaddles]->Acquire();
		if (FAILED (hr))	
			return DIENUM_STOP;

		hr=joy.dinPaddle[joy.nbPaddles]->EnumObjects (EnumAxesCallback, (VOID*)joy.hwnd, DIDFT_AXIS );
		if (FAILED (hr))	
			return DIENUM_STOP;

		joy.nbPaddles++;
	}
	else
		return DIENUM_STOP;

	return DIENUM_CONTINUE;
}


/**
 *  Enum axes Callback
 */
BOOL CALLBACK EnumAxesCallback( const DIDEVICEOBJECTINSTANCE* pdidoi, VOID* pContext )
{
	JoypadManager &joy = JoypadManager::getInstance();

	HWND hDlg = (HWND)pContext;
	DIPROPRANGE diprg; 
	diprg.diph.dwSize       = sizeof(DIPROPRANGE); 
	diprg.diph.dwHeaderSize = sizeof(DIPROPHEADER); 
	diprg.diph.dwHow        = DIPH_BYID; 
	diprg.diph.dwObj        = pdidoi->dwType;
	diprg.lMin              = -1000; 
	diprg.lMax              = +1000; 
    
	if( FAILED( joy.dinPaddle[joy.nbPaddles]->SetProperty( DIPROP_RANGE, &diprg.diph ) ) )
		return DIENUM_STOP;


    return DIENUM_CONTINUE;
}

/**
 * Delete
 */
void JoypadManager::clean()
{
	for( int i=0; i<nbPaddles; i++ )
	{
		if (din != NULL && dinPaddle[i] != NULL)
		{
			dinPaddle[i]->Unacquire();
			dinPaddle[i]->Release();
			dinPaddle[i] = NULL;
		}
	}

}

#endif