#ifndef __LightManager__
#define __LightManager__

#include <vector>
#include "Singleton.h"
#include "DirectXManager.h"

/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class LightManager : public Singleton<LightManager>
{
	private:

		friend class Singleton<LightManager>; 
		int nb_light_max;
		D3DLIGHT9 ** tab_light;

	public :
		/*    
		D3DLIGHTTYPE    Type;             // Type of light source (D3DLIGHT_POINT, D3DLIGHT_SPOT, D3DLIGHT_DIRECTIONAL)
		D3DCOLORVALUE   Diffuse;          // Diffuse color of light 
		D3DCOLORVALUE   Specular;         // Specular color of light 
		D3DCOLORVALUE   Ambient;          // Ambient color of light 
		D3DVECTOR       Position;         // Position in world space 
		D3DVECTOR       Direction;        // Direction in world space 
		float           Range;            // Cutoff range 
		float           Falloff;          // Falloff 
		float           Attenuation0;     // Constant attenuation 
		float           Attenuation1;     // Linear attenuation 
		float           Attenuation2;     // Quadratic attenuation 
		float           Theta;            // Inner angle of spotlight cone 
		float           Phi;			  // Outer angle of spotlight cone
		*/

		void init(int _nb_light_max=8);
		int addLightDirectional(const D3DCOLORVALUE &, const D3DCOLORVALUE &, const D3DCOLORVALUE &);
		int addLightPoint(const D3DCOLORVALUE &, const D3DCOLORVALUE &, const D3DCOLORVALUE &, const D3DVECTOR &, float , float , float , float );
		int addLightSpot(const D3DCOLORVALUE &, const D3DCOLORVALUE &, const D3DCOLORVALUE &, const D3DVECTOR &, const D3DVECTOR &, float , float , float , float , float , float , float );

		void enableLight(int i);
		void disenableLight(int i);
		D3DLIGHT9* getLight(int i);
  
		void setDiffuse(int i, const D3DCOLORVALUE &);
		void setSpecular(int i, const D3DCOLORVALUE &);
		void setAmbient(int i, const D3DCOLORVALUE &);
		void setPosition(int i, const D3DVECTOR &);
		void setDirection(int i, const D3DVECTOR &);
		void setRange(int i, float);
		void setFalloff(int i, float);
		void setAttenuation0(int i, float );
		void setAttenuation1(int i, float);
		void setAttenuation2(int i, float);
		void setTheta(int i, float);
		void setPhi(int i, float);

		void delLight(int i);
		void clear();
		~LightManager();

	private:
		LightManager();
		int searchEmpty();


};

/**
 * Private constructor
 */
LightManager::LightManager()
{
}

/**
 * Private function search an empty place in tab_light;
 */
int LightManager::searchEmpty()
{
	int result = -1;
	for(int i=0 ; i < nb_light_max && result == -1 ; ++i)
	{
		if(!tab_light[i])
			result = i;
	}
	return result;
}

/**
 * public Destructor
 */
LightManager::~LightManager()
{
	clear();
	delete [] tab_light;
}

/**
 * Init the light manager
 * @param int _nb_light_max
 */
void LightManager::init(int _nb_light_max)
{
	nb_light_max = _nb_light_max;
	tab_light = new D3DLIGHT9* [nb_light_max];

	for(int i=0 ; i < nb_light_max ; ++i)
		if(tab_light[i])
			tab_light[i] = NULL;
}

/**
 * Add a light Directional in manager
 * @param const D3DCOLORVALUE & _diffuse
 * @param const D3DCOLORVALUE & _specular
 * @param const D3DCOLORVALUE & _ambient
 */
int LightManager::addLightDirectional(const D3DCOLORVALUE & _diffuse, const D3DCOLORVALUE & _specular, const D3DCOLORVALUE & _ambient)
{
	DirectXManager & dxm = DirectXManager::getInstance();

	int pos_tab = searchEmpty();
	if(pos_tab != -1)
	{
		tab_light[pos_tab] = new D3DLIGHT9;
		ZeroMemory(tab_light[pos_tab], sizeof(D3DLIGHT9));

		tab_light[pos_tab]->Type		= D3DLIGHT_DIRECTIONAL; 
		tab_light[pos_tab]->Diffuse		= _diffuse;
		tab_light[pos_tab]->Specular	= _specular;
		tab_light[pos_tab]->Ambient		= _ambient;

		dxm.d3ddev->SetLight(pos_tab, tab_light[pos_tab]);
		dxm.d3ddev->LightEnable(pos_tab, TRUE);
	}

	return pos_tab;
}

/**
 * Add a light point in manager
 * @param const D3DCOLORVALUE & _diffuse
 * @param const D3DCOLORVALUE & _specular
 * @param const D3DCOLORVALUE & _ambient
 * @param const D3DVECTOR & _position
 * @param float _range
 * @param float _attenuation0
 * @param float _attenuation1
 * @param float _attenuation2
 */
int LightManager::addLightPoint(const D3DCOLORVALUE & _diffuse, const D3DCOLORVALUE & _specular, const D3DCOLORVALUE & _ambient, const D3DVECTOR & _position, float _range, float _attenuation0, float _attenuation1, float _attenuation2)
{
	DirectXManager & dxm = DirectXManager::getInstance();

	int pos_tab = searchEmpty();
	if(pos_tab != -1)
	{	
		tab_light[pos_tab] = new D3DLIGHT9;
		ZeroMemory(tab_light[pos_tab], sizeof(D3DLIGHT9));

		tab_light[pos_tab]->Type		= D3DLIGHT_POINT; 
		tab_light[pos_tab]->Diffuse		= _diffuse;
		tab_light[pos_tab]->Specular	= _specular;
		tab_light[pos_tab]->Ambient		= _ambient;
		tab_light[pos_tab]->Position	= _position;

		
		tab_light[pos_tab]->Range		 = _range;
		tab_light[pos_tab]->Attenuation0 = _attenuation0;
		tab_light[pos_tab]->Attenuation1 = _attenuation1;
		tab_light[pos_tab]->Attenuation2 = _attenuation2;

		dxm.d3ddev->SetLight(pos_tab, tab_light[pos_tab]);
		dxm.d3ddev->LightEnable(pos_tab, TRUE);
	}

	return pos_tab;
}

/**
 * Add a light spot in manager
 * @param const D3DCOLORVALUE & _diffuse
 * @param const D3DCOLORVALUE & _specular
 * @param const D3DCOLORVALUE & _ambient
 * @param const D3DVECTOR & _position
 * @param float _range
 * @param float float _falloff
 * @param float _attenuation0
 * @param float _attenuation1
 * @param float _attenuation2
 * @param float _theta
 * @param float _phi
 */
int LightManager::addLightSpot(const D3DCOLORVALUE & _diffuse, const D3DCOLORVALUE & _specular, const D3DCOLORVALUE & _ambient, const D3DVECTOR & _position, const D3DVECTOR & _direction,
				 float _range, float _falloff, float _attenuation0, float _attenuation1, float _attenuation2, float _theta, float _phi)
{
	DirectXManager & dxm = DirectXManager::getInstance();

	int pos_tab = searchEmpty();
	if(pos_tab != -1)
	{
		tab_light[pos_tab] = new D3DLIGHT9;
		ZeroMemory(tab_light[pos_tab], sizeof(D3DLIGHT9));

		tab_light[pos_tab]->Type		 = D3DLIGHT_SPOT; 
		tab_light[pos_tab]->Diffuse		 = _diffuse;
		tab_light[pos_tab]->Specular	 = _specular;
		tab_light[pos_tab]->Ambient		 = _ambient;
		tab_light[pos_tab]->Position	 = _position;
		tab_light[pos_tab]->Direction	 = _direction;
		tab_light[pos_tab]->Range		 = _range;
		tab_light[pos_tab]->Falloff		 = _falloff;
		tab_light[pos_tab]->Attenuation0 = _attenuation0;
		tab_light[pos_tab]->Attenuation1 = _attenuation1;
		tab_light[pos_tab]->Attenuation2 = _attenuation2;
		tab_light[pos_tab]->Theta		 = _theta;
		tab_light[pos_tab]->Phi			 = _phi;


		dxm.d3ddev->SetLight(pos_tab, tab_light[pos_tab]);
		dxm.d3ddev->LightEnable(pos_tab, TRUE);
	}

	return pos_tab;
}

/**
 * Delete a light in manager
 * @param int i
 */
void LightManager::delLight(int i)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i] )
	{
		delete tab_light[i];
		tab_light[i] = NULL;
		dxm.d3ddev->LightEnable(i, FALSE);
	}
}

/**
 * Enable a light
 * @param int i
 */
void LightManager::enableLight(int i)
{	
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i] )
	{
		dxm.d3ddev->LightEnable(i, TRUE);
	}
}

/**
 * Disenable a light
 * @param int i
 */
void LightManager::disenableLight(int i)
{	
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i] )
	{
		dxm.d3ddev->LightEnable(i, FALSE);
	}
}

/**
 * Destroy all lights in manager
 */
void LightManager::clear()
{
	DirectXManager & dxm = DirectXManager::getInstance();
	for(int i=0 ; i < nb_light_max ; ++i )
		if( tab_light[i] )
		{
			delete tab_light[i];
			tab_light[i] = NULL;
			dxm.d3ddev->LightEnable(i, FALSE);
		}
}

/**
 * get a light in manager
 * but return NULL if the light doesn't exist
 * @param int i
 */
D3DLIGHT9* LightManager::getLight(int i)
{
	D3DLIGHT9 * result = NULL;
	if( i < nb_light_max )
		result = tab_light[i];
	return result;
}

/**
 * set diffuse of light
 * @param int i
 * @param const D3DCOLORVALUE & _diffuse
 */
void LightManager::setDiffuse(int i, const D3DCOLORVALUE & _diffuse)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Diffuse = _diffuse;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set specular of light
 * @param int i
 * @param const D3DCOLORVALUE & _specular
 */
void LightManager::setSpecular(int i, const D3DCOLORVALUE & _specular)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Specular = _specular;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set ambient of light
 * @param int i
 * @param const D3DCOLORVALUE & _ambient
 */
void LightManager::setAmbient(int i, const D3DCOLORVALUE & _ambient)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Ambient = _ambient;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set position of light
 * @param int i
 * @param const D3DVECTOR & _position
 */
void LightManager::setPosition(int i, const D3DVECTOR & _position)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Position = _position;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set direction of light
 * @param int i
 * @param const D3DVECTOR & _direction
 */
void LightManager::setDirection(int i, const D3DVECTOR & _direction)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Direction = _direction;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set range of light
 * @param int i
 * @param float _range
 */
void LightManager::setRange(int i, float _range)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Range = _range;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set falloff light
 * @param int i
 * @param float _falloff
 */
void LightManager::setFalloff(int i, float _falloff)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Falloff = _falloff;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set attenuation0 of light
 * @param int i
 * @param float _attenuation0
 */
void LightManager::setAttenuation0(int i, float _attenuation0)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Attenuation0 = _attenuation0;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set _attenuation1 of light
 * @param int i
 * @param float _attenuation1
 */
void LightManager::setAttenuation1(int i, float _attenuation1)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Attenuation1 = _attenuation1;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set _attenuation2 of light
 * @param int i
 * @param float _attenuation2
 */
void LightManager::setAttenuation2(int i, float _attenuation2)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Attenuation2 = _attenuation2;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set theta of light
 * @param int i
 * @param float _theta
 */
void LightManager::setTheta(int i, float _theta)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Theta = _theta;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

/**
 * set phi of light
 * @param int i
 * @param float _phi
 */
void LightManager::setPhi(int i, float _phi)
{
	DirectXManager & dxm = DirectXManager::getInstance();
	if( i < nb_light_max && tab_light[i])
	{
		tab_light[i]->Phi = _phi;
		dxm.d3ddev->SetLight(i, tab_light[i]);
	}
}

#endif